/* Copyright (c) 2005. Chris Richardson. All rights reserved */

package net.chrisrichardson.bankingExample.facade.spring;

import junit.framework.*;
import net.chrisrichardson.bankingExample.domain.*;
import net.chrisrichardson.bankingExample.facade.*;
import net.chrisrichardson.foodToGo.util.spring.*;

import org.springframework.beans.factory.*;
import org.springframework.beans.factory.access.*;
import org.springframework.context.access.*;

public class SpringHibernateAccountManagementFacadeTests extends TestCase {

    private AccountManagementFacade facade;

    protected void setUp() {
        facade = (AccountManagementFacade) TheBeanFactory.getBean(
                "configurations/banking-context.xml", "spring-context.xml",
                "AccountManagementFacade", AccountManagementFacade.class);
    }

    public void test() throws Exception {

        String accountId = "A." + System.currentTimeMillis();
        double initialBalance = 1.5;
        Account account = facade.createAccount(accountId, initialBalance);
        assertEquals(accountId, account.getAccountId());

        double balance = facade.getBalance(accountId);
        assertEquals(initialBalance, balance);
    }
    
    public void testTransfer() throws Exception {
        String accountId1 = "A." + System.currentTimeMillis();
        double initialBalance1 = 10;
        String accountId2 = "B." + System.currentTimeMillis();
        double initialBalance2 = 20;
        Account account1 = facade.createAccount(accountId1, initialBalance1);
        Account account2 = facade.createAccount(accountId2, initialBalance2);
        facade.transfer(accountId1, accountId2, 5);
        
        double balance1 = facade.getBalance(accountId1);
        double balance2 = facade.getBalance(accountId2);
        
        assertEquals(initialBalance1 - 5, balance1);
        assertEquals(initialBalance2 + 5, balance2);
        
    }

}
