/* Copyright (c) 2005. Chris Richardson. All rights reserved */

package net.chrisrichardson.ormunit.hibernate;

import java.lang.reflect.*;
import java.util.*;
import java.util.Set;

import junit.framework.*;

import org.hibernate.mapping.*;

public class HibernateAssertUtil {

    public static void assertColumn(String idColumn,
            Value identifier) {
        Iterator columnIt = identifier.getColumnIterator();
        Assert.assertTrue(columnIt.hasNext());
        Column column = (Column) columnIt.next();
        Assert.assertFalse(columnIt.hasNext());
        if (!idColumn.equalsIgnoreCase(column.getName()))
            Assert.assertEquals(idColumn, column.getName());
    }

    public static void assertPropertyColumn(String idColumn,
            Property idProperty) {
        Value identifier = idProperty.getValue();
        assertColumn(idColumn, identifier);
    }

    public static Set getPersistableFields(Class type) {
        Set persistableFields = new HashSet();
        Field[] fields = type.getDeclaredFields();
        Set unmappedFields = new HashSet();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            String fieldName = field.getName();
            if (isPersistableField(field)) {
                persistableFields.add(field.getName());
            }
        }
        return persistableFields;
    }

    public static boolean isPersistableField(Field field) {
        return (field.getModifiers() & (Modifier.FINAL | Modifier.STATIC)) == 0;
    }

    public static void assertFieldAccess(Property property) {
        Assert.assertEquals("field", property
                .getPropertyAccessorName());
    }

}