/* Copyright (c) 2005. Chris Richardson. All rights reserved */

package net.chrisrichardson.ormunit.hibernate;

import java.util.*;
import java.util.Set;

import junit.framework.*;
import org.hibernate.*;
import org.hibernate.cfg.*;
import org.hibernate.mapping.*;

public abstract class HibernateMappingTests extends HibernateORMTestCase {

    private PersistentClass classMapping;

    private Class type;

    protected void setUp() throws Exception {
        super.setUp();
        this.type = null;
        this.classMapping = null;
    }

    protected final String getConfigXML() { return null; }

    protected ComponentFieldMapping getComponentFieldMapping(String fieldName) throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        Component value = (Component) property.getValue();
        return new ComponentFieldMapping(property, value);
    }

    protected void assertAllFieldsMapped() throws MappingException {
        assertNonPersistentFields(Collections.EMPTY_SET);
    }
    
    protected void assertNonPersistentFields(Set fieldNames)
            throws MappingException {
        assertNoMappingForFields(fieldNames);
        assertNoOtherUnmappedFields(fieldNames);
    }

    private void assertNoOtherUnmappedFields(Set nonPersistentFields)
            throws MappingException {
        Property idProperty = classMapping.getIdentifierProperty();
        Set unmappedFields = new HashSet();
        Set fieldsToIgnore = new HashSet(nonPersistentFields);
        fieldsToIgnore.add(idProperty.getName());
        for (Iterator it = HibernateAssertUtil.getPersistableFields(type).iterator(); it.hasNext();) {
            String fieldName = (String) it.next();
            if (!fieldsToIgnore.contains(fieldName))
                try {
                    classMapping.getProperty(fieldName);
                } catch (MappingException e) {
                    unmappedFields.add(fieldName);
                }
        }
        if (!unmappedFields.isEmpty())
            fail("unmapped fields: " + unmappedFields);
    }

    private void assertNoMappingForFields(Set fieldNames) {
        for (Iterator iter = fieldNames.iterator(); iter.hasNext();) {
            String fieldName = (String) iter.next();
            try {
                classMapping.getProperty(fieldName);
                fail("Unexpected mapping for field: " + fieldName);
            } catch (MappingException e) {

            }
        }
    }

    protected void assertCompositeListField(String fieldName)
            throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        Value v = property.getValue();
        org.hibernate.mapping.List value = (org.hibernate.mapping.List) v;
    }

    protected CompositeListFieldMapping getCompositeListFieldMapping(String fieldName) throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        Value v = property.getValue();
        org.hibernate.mapping.List value = (org.hibernate.mapping.List) v;
        return new CompositeListFieldMapping(property, value);
    }
    
    protected void assertComponentField(String fieldName) throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        Component value = (Component) property.getValue();
    }

    protected void assertSetField(String fieldName) throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        org.hibernate.mapping.Set value = (org.hibernate.mapping.Set) property.getValue();
    }

    protected void assertManyToOneField(String fieldName, String foreignKeyColumnName) throws MappingException {
        Property subProperty = classMapping.getProperty(fieldName);
        ManyToOne value = (ManyToOne) subProperty.getValue();
        HibernateAssertUtil.assertColumn(foreignKeyColumnName, value);
    }
 
    protected void assertVersionField(String fieldName,
            String columnName) {
        Property versionProperty = classMapping.getVersion();
        assertEquals(fieldName, versionProperty.getName());
        HibernateAssertUtil.assertPropertyColumn(columnName, versionProperty);
    }

    protected void assertClassMapping(Class type, String tableName) {
        this.type = type;
        classMapping = getConfiguration().getClassMapping(type.getName());
        assertEquals(tableName, classMapping.getTable().getName());
    }

    protected void assertIdField(String idField, String idColumn)
            throws PropertyNotFoundException, MappingException {
        Property idProperty = classMapping.getIdentifierProperty();
        HibernateAssertUtil.assertFieldAccess(idProperty);
    }

    public void assertField(String fieldName, String columnName) throws MappingException {
        Property subProperty = classMapping.getProperty(fieldName);
        HibernateAssertUtil.assertPropertyColumn(columnName, subProperty);
    }

    protected void assertOneToManyListField(String fieldName, String foreignKeyColumn, String indexColumn) throws MappingException {
        Property property = classMapping.getProperty(fieldName);
        org.hibernate.mapping.List value = (org.hibernate.mapping.List) property.getValue();
        HibernateAssertUtil.assertColumn(foreignKeyColumn, value.getKey());
        HibernateAssertUtil.assertColumn(indexColumn, value.getIndex());
        assertTrue(value.getElement() instanceof OneToMany);
    }
    
    protected void assertNonPersistentFields(String field1, String field2, String field3) throws MappingException {
        Set s = new HashSet();
        s.add(field1);
        s.add(field2);
        s.add(field3);
        assertNonPersistentFields(s);
    }

    protected void assertNonPersistentFields(String field1, String field2) throws MappingException {
        Set s = new HashSet();
        s.add(field1);
        s.add(field2);
        assertNonPersistentFields(s);
    }

    protected void assertNonPersistentField(String field1) throws MappingException {
        Set s = new HashSet();
        s.add(field1);
        assertNonPersistentFields(s);
    }
    
    public CompositeSetFieldMapping getCompositeSetFieldMapping(String fieldName) {
        Property property = classMapping.getProperty(fieldName);
        Value v = property.getValue();
        org.hibernate.mapping.Set value = (org.hibernate.mapping.Set) v;
        return new CompositeSetFieldMapping(property, value);
    }

}