/* Copyright (c) 2005. Chris Richardson. All rights reserved */

package net.chrisrichardson.ormunit.hibernate;

import java.sql.*;
import java.util.*;

import junit.framework.*;
import net.chrisrichardson.foodToGo.util.hibernate.*;
import net.chrisrichardson.foodToGo.util.spring.*;
import org.hibernate.*;
import org.hibernate.cfg.*;

import org.springframework.orm.hibernate3.*;
import org.springframework.transaction.*;
import org.springframework.transaction.support.*;

public class HibernatePersistenceTests extends HibernateORMTestCase {

    protected SessionFactory sessionFactory;

    private HibernateTransactionManager transactionManager;

    private TransactionTemplate transactionTemplate;

    protected HibernateTemplate hibernateTemplate;

    protected void setUp() throws Exception {
        sessionFactory = getSessionFactory();

        transactionManager = new HibernateTransactionManager(
                sessionFactory);
        transactionTemplate = new TransactionTemplate(
                transactionManager);
        hibernateTemplate = new HibernateTemplate(sessionFactory);
    }

    public void save(Object r) {
        hibernateTemplate.save(r);
    }

    public Object load(Class type, String id) {
        return hibernateTemplate.get(type, new Integer(id));
    }

    public void doWithTransaction(final TxnCallback cb) {
        transactionTemplate
                .execute(new TransactionCallbackWithoutResult() {

                    protected void doInTransactionWithoutResult(
                            TransactionStatus status) {
                        try {
                            cb.execute();
                        } catch (RuntimeException e) {
                            throw e;
                        } catch (Throwable e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    }

    public HibernateTemplate getHibernateTemplate() {
        return hibernateTemplate;
    }

    public String getId(final Object object) {
        return (String) hibernateTemplate
                .execute(new HibernateCallback() {

                    public Object doInHibernate(Session session)
                            throws HibernateException {
                        return session.getIdentifier(object)
                                .toString();
                    }
                });
    }

    protected void delete(Class type) {
        HibernateTemplate t = getHibernateTemplate();
        t.deleteAll(t.find("from " + type.getName()));
    }

    protected void delete(Object object) {
        getHibernateTemplate().delete(object);
    }

    protected String getIdentifier(final Object object) {
        return (String)getHibernateTemplate().execute(new HibernateCallback(){
    
            public Object doInHibernate(Session session) throws HibernateException, SQLException {
                return session.getIdentifier(object);
            }});
    }
}