/* Copyright (c) 2005. Chris Richardson. All rights reserved */

package net.chrisrichardson.ormunit.hibernate;

import java.util.*;

import org.hibernate.*;
import org.hibernate.cfg.*;
import org.hibernate.dialect.*;
import org.hibernate.tool.hbm2ddl.*;

public abstract class HibernateSchemaTests extends
        HibernateORMTestCase {

    public void assertDatabaseSchema()
            throws Exception {
        String[] script = generateScript();
        List differences = getSignificantDifferences(script);
        System.out.println(differences);
        assertTrue(differences.toString(), differences
                .isEmpty());
    }

    private String[] generateScript()
            throws Exception {
        Configuration cfg = getConfiguration();
        SessionFactory sessionFactory = getSessionFactory();
        Session session = sessionFactory.openSession();
        try {
            Dialect dialect = getDatabaseDialect();
            DatabaseMetadata dbm = new DatabaseMetadata(
                    session.connection(), dialect);
            String[] script = cfg
                    .generateSchemaUpdateScript(
                            dialect, dbm);
            return script;
        } finally {
            session.close();
        }
    }

    protected Dialect getDatabaseDialect() throws Exception {
        return (Dialect)Class.forName(
                getConfiguration().getProperty(
                        "hibernate.dialect"))
                .newInstance();
    }

    private List getSignificantDifferences(
            String[] script) {
        List differences = new ArrayList();
        for (int i = 0; i < script.length; i++) {
            String line = script[i];
            if (line.indexOf("add constraint") == -1)
                differences.add(line);
        }
        return differences;
    }

}